#ifndef _STDLIB_H_
#define _STDLIB_H_

#include <sys/types.h>
#include <malloc.h>

/* Don't overwrite user definitions of NULL */
#ifndef NULL
#define NULL ((void *)0)
#endif

/* Returned by `div' */
typedef struct {
	int	quot;		/* Quotient */
	int	rem;		/* Remainder */
} div_t;

/* Returned by `ldiv' */
typedef struct {
	long int quot;		/* Quotient */
	long int rem;		/* Remainder */
} ldiv_t;

/* For program termination */
#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0

extern void _exit(int);
extern void exit(int);
extern void abort(void);

extern unsigned int sleep(unsigned int seconds);

extern int rand(void);
extern void srand(uint seed);

extern char *__longtoa(unsigned long, char *, int, char, char);
extern char *itoa(int value, char *strP, int radix);
extern char *ultoa(unsigned long value, char *strP, int radix);
extern char *ltoa(long value, char *strP, int radix);

extern int atoi(char *str);
extern long atol(char *str);

extern char *_itoa(int value);
extern char *_ltoa(long value);
extern char *_ultoa(unsigned long value);

extern char *ultostr(unsigned long value, int radix);
extern char *ltostr(long value, int radix);

extern long strtol(char *nptr, char **endptr, int base);
extern unsigned long strtoul(char *nptr, char **endptr, int base);
#ifndef __HAS_NO_FLOATS__
extern double strtod(char *nptr, char **endptr);
#endif

extern char **environ;
extern char *getenv(char *);
extern int putenv(char *);
extern int setenv(char *, char *, int);
extern void unsetenv(char *);

typedef void (*atexit_t)(void);
typedef void (*onexit_t)(int, void *);
extern int atexit(atexit_t);
extern int on_exit(onexit_t, void *arg);
extern onexit_t __cleanup;

extern char *crypt(char *__key, char *__salt);

typedef int (*cmp_func_t)(void *, void *);

extern int _bsearch;
extern void *bsearch(void *key, void *base, size_t num, size_t size, cmp_func_t cmp);
extern void *lfind(void *key, void *base, size_t *num, size_t size, cmp_func_t cmp);
extern void *lsearch(void *key, void *base, size_t *num, size_t size, cmp_func_t cmp);
extern void *_qbuf;
extern void qsort(void *base, size_t num, size_t size, cmp_func_t cmp);

extern int opterr;
extern int optind;
extern char *optarg;
extern int getopt(int argc, char *argv[], char *optstring);

extern char *getpass(char *prompt);

extern int _argc;
extern char **_argv;

#endif /* __STDLIB_H */
