/*
 * Punix, Puny Unix kernel
 * Copyright 2005 Chris Williams
 * 
 * $Id: syscall.c,v 1.2 2005/03/28 08:50:32 chris Exp $
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*-
 * Copyright (c) 1986, 1989, 1991, 1993
 *	The Regents of the University of California.  All rights reserved.
 * (c) UNIX System Laboratories, Inc.
 * All or some portions of this file are derived from material licensed
 * to the University of California by American Telephone and Telegraph
 * Co. or Unix System Laboratories, Inc. and are reproduced herein with
 * the permission of UNIX System Laboratories, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <sys/proc.h>
#include <sys/sysent.h>

#include <sys/punix.h>
#include <sys/process.h>

/* Processor Status word bit states */
#define PS_C	0x0001	/* carry */
#define PS_V	0x0002	/* overflow */
#define PS_Z	0x0004	/* zero */
#define PS_N	0x0008	/* negative */
#define PS_X	0x0010	/* extend */
#define PS_IPL	0x0700	/* interrupt priority level */
#define PS_SUP	0x2000	/* supervisor mode */
#define PS_T	0x8000	/* trace */

extern int copyin(void *dest, void *src, int count);

/*
 * system call function
 * upon return, carry indicates error
 */
STARTUP(unsigned long syscall(void **sp, unsigned int callno, volatile int ps, void *pc))
{
	extern const int nsysent;
	struct sysent *callp;
	long retval = 0;
	
	/* get sysent[r] */
	if (callno >= nsysent)
		callno = 0;
	callp = &sysent[callno];
	
	/* copy arguments from user stack to argument buffer in user struct */
	if (callp->sy_narg) /* &sp[1] below skips over return address */
		copyin(P.p_arg, &sp[1], callp->sy_narg*sizeof(int));
	
	do {
		P.p_error =
		P.p_retval = 0;
		callp->sy_call();
	} while (P.p_error == ERESTART);
	/* ERESTART is a kernel-only error code */
	
	switch (P.p_error) {
	case 0: /* no error */
		ps &= ~PS_C; /* clear carry */
		retval = P.p_retval;
		break;
#if 0
	case ERESTART:
		/* do it again somehow
		 (either pc = old pc or goto again) */
		break;
#endif
	case EJUSTRETURN:
		break;
	default:
		/* return error */
		ps |= PS_C; /* set carry */
		retval = P.p_error;
		break;
	}
	
	/* post all signals here */
	
	return retval;
}

STARTUP(void sys_NONE(void))
{
	P.p_error = ENOSYS;
}
