/*
 * Punix, Puny Unix kernel
 * Copyright 2005 Chris Williams
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Copyright (c) 1987,1997, Prentice Hall
 * All rights reserved.
 * 
 * Redistribution and use of the MINIX operating system in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *    * Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 * 
 *    * Neither the name of Prentice Hall nor the names of the software
 *      authors or contributors may be used to endorse or promote
 *      products derived from this software without specific prior
 *      written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS, AUTHORS, AND
 * CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PRENTICE HALL OR ANY AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PROC_H_
#define _PROC_H_

/*#include "type.h"*/
#include <time.h>
#include <signal.h>
#include <sys/types.h>

#include "procglo.h"
#include "file.h"

#define NR_PROC	20

typedef long register_t;		/* machine register */

struct stackframe {
	register_t retreg /*d0*/, d1, d2, d3, d4, d5, d6, d7;
	register_t a0, a1, a2, a3, a4, a5, fp /*a6*/, sp /*a7*/;
	register_t pc;
	u_int16_t psw;
	u_int16_t dummy;		/* make size multiple of reg_t for system.c */
};

struct fsave {
  struct cpu_state {
	u_int16_t i_format;
	u_int32_t i_addr;
	u_int16_t i_state[4];
  } cpu_state;
  struct state_frame {
	u_int8_t frame_type;
	u_int8_t frame_size;
	u_int16_t reserved;
	u_int8_t frame[212];
  } state_frame;
  struct fpp_model {
	u_int32_t fpcr;
	u_int32_t fpsr;
	u_int32_t fpiar;
	struct fpN {
		u_int32_t high;
		u_int32_t low;
		u_int32_t mid;
	} fpN[8];
  } fpp_model;
};

struct proc {
	struct proc *p_next;
	char p_status;	/* stopped, ready, running, zombie, etc. */
	char p_insys;
	
	/* scheduling */
	char p_nice;
	signed char p_pri;	/* priority */
	signed char p_cpu;	/* amount of time we get each timeslice (in ticks) */
	char dummy;
	
	/* processor state */
	struct stackframe p_reg;	/* process's registers saved in stack frame */
	void *p_sp;			/* saved SP for context switches */
	
	/* memory */
#if 0
#if (CHIP == M68000)
	register_t p_splow;		/* lowest observed stack value */
	int p_trap;		/* trap type (only low byte) */
#if (SHADOWING == 0)
	char *p_crp;		/* mmu table pointer (really struct _rpr *) */
#else
	phys_clicks p_shadow;	/* set if shadowed process image */
	int align;		/* make the struct size a multiple of 4 */
#endif
	char p_physio;		/* cannot be (un)shadowed now if set */
#if defined(FPP)
	struct fsave p_fsave;	/* FPP state frame and registers */
	int align2;		/* make the struct size a multiple of 4 */
#endif
#endif /* (CHIP == M68000) */
#endif /* 0 */
	
	
	register_t *p_stguard;	/* stack guard word */
	
	/* id */
	pid_t p_pid;		/* process id */
	uid_t p_uid;
	pid_t p_pgrp;
	struct proc *p_pptr;	/* parent proc */
	
	/* signals */
	int p_sigmask;	/* correct type ? */
	int p_sigcatch;	/* correct type ? */
	int p_sigignore;	/* correct type ? */
	
	/* syscalls */
	int p_arg[8];	/* probably don't need 8 words */
	long p_retval;
	int p_error;
	
	/* files */
	struct file *p_ofile[NR_OPEN];	/* file structures for open files */
	char p_ofileflags[NR_OPEN];	/* flags of open files */
	int p_lastfile;
#define	FD_CLOEXEC	1	/* close on exec */
#if 0
#define	UF_MAPPED	2	/* mapped from device */
#endif
	
#if 0
	clock_t user_time;	/* user time in ticks */
	clock_t sys_time;	/* sys time in ticks */
	clock_t child_utime;	/* cumulative user time of children */
	clock_t child_stime;	/* cumulative sys time of children */
	clock_t p_alarm;	/* time of next alarm in ticks, or 0 */
#endif
	
//	struct proc *p_nextready;	/* pointer to next ready process */
	sigset_t p_pending;	/* bit map for pending signals */
//	unsigned p_pendcount;	/* count of pending and unfinished signals */
	
	char p_name[16];	/* name of the process */
	
	/* zombie */
	short p_xstat;		/* exit status */
};

/* are these all good? */
#define P_FREE		0	/* slot is empty */
#define P_RUNNING	1	/* running */
#define P_SLEEPING	2	/* waiting for an event */
#define P_STOPPED	3	/**/
#define P_TRACED	4
#define P_ZOMBIE	5

#if 0
/* Guard word for task stacks. */
#define STACK_GUARD	((reg_t) (sizeof(reg_t) == 2 ? 0xBEEF : 0xDEADBEEF))

/* Bits for p_flags in proc[].  A process is runnable iff p_flags == 0. */
#define P_SLOT_FREE      001	/* set when slot is not in use */
#define NO_MAP           002	/* keeps unmapped forked child from running */
#define SENDING          004	/* set when process blocked trying to send */
#define RECEIVING        010	/* set when process blocked trying to recv */
#define PENDING          020	/* set when inform() of signal pending */
#define SIG_PENDING      040	/* keeps to-be-signalled proc from running */
#define P_STOP		0100	/* set when process is being traced */

/* Magic process table addresses. */
#define BEG_PROC_ADDR (&proc[0])
#define END_PROC_ADDR (&proc[NR_PROCS])
#define BEG_USER_ADDR (&proc[LOW_USER])

#define NIL_PROC          ((struct proc *) 0)
#define isidlehardware(n) ((n) == IDLE || (n) == HARDWARE)
#define isokprocn(n)      ((unsigned) ((n) + NR_TASKS) < NR_PROCS + NR_TASKS)
#define isoksrc_dest(n)   (isokprocn(n) || (n) == ANY)
#define isoksusern(n)     ((unsigned) (n) < NR_PROCS)
#define isokusern(n)      ((unsigned) ((n) - LOW_USER) < NR_PROCS - LOW_USER)
#define isrxhardware(n)   ((n) == ANY || (n) == HARDWARE)
#define issysentn(n)      ((n) == FS_PROC_NR || (n) == MM_PROC_NR)
#define istaskp(p)        ((p) < END_TASK_ADDR && (p) != proc_addr(IDLE))
#define isuserp(p)        ((p) >= BEG_USER_ADDR)
#define proc_addr(n)      (pproc_addr)[(n)]
#define cproc_addr(n)     (&proc[(n)])
#define proc_number(p)    ((p)->p_nr)
#define proc_vir2phys(p, vir) \
			  (((phys_bytes)(p)->p_map[D].mem_phys << CLICK_SHIFT) \
							+ (vir_bytes) (vir))
#if (SHADOWING == 1)
#define isshadowp(p)      ((p)->p_shadow != 0)
#endif

//extern struct proc proc[NR_PROCS];	/* process table */
extern struct proc *pproc_addr[NR_PROCS];
/* ptrs to process table slots; fast because now a process entry can be found
   by indexing the pproc_addr array, while accessing an element i requires
   a multiplication with sizeof(struct proc) to determine the address */
extern struct proc *bill_ptr;	/* ptr to process to bill for clock ticks */
extern struct proc *rdy_head;	/* pointers to ready list headers */
extern struct proc *rdy_tail;	/* pointers to ready list tails */
//extern struct proc *current;
#endif

#endif /* PROC_H */
